unit Slides;

{ A simple Web page click animation entirely done       }
{ in Canterbury Pascal for Java.                        }
{ Last update: March 17, 2000                           }
{                                                       }
{ (c) 1998 by Mill Hill & Canterbury Corporation, Ltd.  }


interface

  uses
    java_applet_Applet,
    java_lang_Runnable,
    java_lang_Thread,
    java_awt_Image,
    java_awt_Graphics,
    java_awt_Event,
    java_lang_RuntimeException;

  type
    ClickAnimation = class( Applet, Runnable )
      protected
        animationThread : Thread;
        cur : integer;
        frame : array [0..3] of Image;
        mouseClick : boolean;
      public
        constructor construct; virtual;
        procedure Init; override;
        procedure start; override;
        procedure stop; override;
        procedure run; virtual;
        procedure paint( g : Graphics ); override;
        function ClickAnimation.mouseDown
        ( e : Event; x : integer; y : integer ) : boolean; override;
    end;

implementation

  uses
    java_lang_String;


  constructor ClickAnimation.construct;
  begin
    inherited _init_;
  end;


  (* The public type-bound procedure "init" overrides the
     one inherited from Java class java.applet.Applet.
     This procedure always gets called first in
     a Java applet and is responsible for intializing
     all the fields
  *)
  procedure ClickAnimation.Init;

    var
      j          : integer;
      gifName    : string;

    function JavaString( s : string ) : java_lang_String.String;
    begin
      NEW( Result, s );
    end;

  begin
    (* set display size for applet *)
    resize(300,300);

    (* get the GIF-formatted images *)
    for j := 0 to 3 do begin
      case j of
        0 :  gifName := 'col1.gif';
        1 :  gifName := 'col2.gif';
        2 :  gifName := 'col3.gif';
        else gifName := 'col4.gif';
      end;
      frame[j] := getImage( getDocumentBase, JavaString(gifName) );
    end;

    (* no mouse click yet *)
    mouseClick := false;
  end;



  (* The public type-bound procedure "start"
     overrides java.applet.Applet.start.
     It is called by the browser or applet viewer to inform this applet
     that it should start its execution.
     It is called after the init method and
     each time the applet is revisited in a Web page.
  *)
  procedure ClickAnimation.start;
  begin
    (* Create a new thread for class ClickAnimation.
       Canterbury Pascal uses an extended standard procedure
       NEW, with an optional second parameter for
       a constrcutor call
    *)
    NEW( animationThread, SELF );

    (* The start() method spawns the animation thread in order
       for the animation to run the first time
    *)
    animationThread.start;

    (* Once the animation thread is started, it calls
       the run() method.
    *)
  end;



  (* The type-bound procedure "stop" overrides java.applet.Applet.stop.
     It is called by the browser or applet viewer to inform this applet
     that it should stop its execution.
     It is called when the Web page that contains this applet has been
     replaced by another page, and also just before the applet is to be
     destroyed.
  *)
  procedure ClickAnimation.stop;
  begin
    try
      animationThread.stop;
    except on RuntimeException do
      { Just catch it.
        Some JVMs, especially in Java 2,
        do a security exception here.
      }
    end;
  end;



  (* The type-bound procedure "run" implements the one from the
     Java interface java.lang.Runnable.
     Starting the thread causes the objects "run" method to be
     called in that separately executing thread.
  *)
  procedure ClickAnimation.run;
  begin
    (* If there was no mouse click repaint the first frame,
       else repaint the next one.
    *)
    if (not mouseClick) then begin
      cur := 0;
      repaint;
    end
    else begin
      mouseClick := false;
      cur := cur + 1;
      if (cur > 3) then begin
        cur := 0;
      end;
      repaint;
    end;
  end;



  (* The type-bound procedure "paint" overrides an inherited one.
     It is responsible for painting the applets display area.
  *)
  procedure ClickAnimation.paint( g : Graphics );
  begin
    g.drawImage( frame[cur], 0, 0, Self );
  end;



  (* Method "mouseDown" overrides an inherited one.
     Always called in case of a mouse button being pressed
     while its cursor is on the applet.
  *)
  function ClickAnimation.mouseDown
    ( e : Event; x : integer; y : integer ) : boolean;
  begin
    mouseClick := true;
    NEW( animationThread, SELF );
    animationThread.start;
    {return} mouseDown := true;
  end;


end.
